
.include "nes.inc"
.include "global.inc"
.import lowlevel_page1, lowlevel_page2, lowlevel_page3, lowlevel_page4

.code
.proc lowlevel_display
  jsr lowlevel_load_palette
  lda #>results_msg
  ldy #<results_msg
  jsr cls_puts_multiline
  jsr lowlevel_display_results
  jsr press_A

  lda #>lowlevel_page1
  ldy #<lowlevel_page1
  jsr cls_puts_multiline
  jsr press_A
  lda #>lowlevel_page2
  ldy #<lowlevel_page2
  jsr cls_puts_multiline
  jsr press_A
  lda #>lowlevel_page3
  ldy #<lowlevel_page3
  jsr cls_puts_multiline
  jsr press_A
  lda #>lowlevel_page4
  ldy #<lowlevel_page4
  jsr cls_puts_multiline
  jmp press_A
.endproc

.proc lowlevel_load_palette
  ; seek to the start of palette memory ($3F00-$3F1F)
  ldx #$3F
  stx PPUADDR
  ldx #$00
  stx PPUADDR
copypalloop:
  lda initial_palette,x
  sta PPUDATA
  inx
  cpx #initial_palette_end-initial_palette
  bcc copypalloop
  rts
.endproc

.define NTXY(tx,ty) ((tx) | ((ty) * 32) | $2000)
PPU_READBACK_DEST = NTXY(7, 10)
PPU_READBACK_LEN = $08
PPU_LATCHTEST_DEST = NTXY(16, 12)
PPU_LATCHTEST_LEN = $05
APU_OPENBUS_DEST = NTXY(22, 14)
APU_OPENBUS_LEN = $03
HEX_P1_DEST = NTXY(9, 20)
HEX_P2_DEST = NTXY(9, 22)
RESULTS_LEN = $04

.proc lowlevel_display_results
  lda #>ppu_readback_values
  sta $05
  lda #<ppu_readback_values
  sta $04
  lda #>PPU_READBACK_DEST
  ldx #<PPU_READBACK_DEST
  ldy #PPU_READBACK_LEN
  jsr hexdump8
  lda #>ppu_latchtest_values
  sta $05
  lda #<ppu_latchtest_values
  sta $04
  lda #>PPU_LATCHTEST_DEST
  ldx #<PPU_LATCHTEST_DEST
  ldy #PPU_LATCHTEST_LEN
  jsr hexdump8
  lda #>open_bus_values
  sta $05
  lda #<open_bus_values
  sta $04
  lda #>APU_OPENBUS_DEST
  ldx #<APU_OPENBUS_DEST
  ldy #APU_OPENBUS_LEN
  jsr hexdump8
  lda #>cbits_p1
  sta $05
  lda #<cbits_p1
  sta $04
  lda #>HEX_P1_DEST
  ldx #<HEX_P1_DEST
  jsr write_results_line
  lda #>cbits_p2
  sta $05
  lda #<cbits_p2
  sta $04
  lda #>HEX_P2_DEST
  ldx #<HEX_P2_DEST
  jsr write_results_line
  rts
.endproc

;;
; @param AAXX nametable destination
; @param $0004-$0005 pointer to four bytes in order 4L 3L 4H 3H
.proc write_results_line

  ; Write hex part
  ldy #RESULTS_LEN
  jsr hexdump8

discipline = $07
src = $04
serialbits = $00
ncbits = $01
minbits = $0F

  ; Form a string listing line disciplines (0, 1, serial, or NC)
  ; in $0007-$000F
  ldy #0
  lda (src),y
  sta minbits
  iny
  eor (src),y
  sta ncbits      ; 4L ^ 3L are NC
  iny
  lda (src),y
  eor minbits
  sta serialbits  ; 4L ^ 4H are serial
  ldy #7
  bitloop:
    lda #0
    lsr serialbits
    rol a
    lsr ncbits
    rol a
    lsr minbits
    rol a
    cmp #4
    bcc :+
      lda #4
    :
    tax
    lda line_discipline_codes,x
    sta discipline,y
    dey
    bpl bitloop
  iny
  sty discipline+8

  ; And write it after the hex result (which ends at the nametable
  ; address in $02)
  sty $01
  lda #<discipline
  sta $00
  lda $03
  ldx $02
  inx
  jmp puts_16
.endproc

.segment "RODATA"
initial_palette:
  .byt $17,$27,$20,$37
initial_palette_end:

line_discipline_codes:  .byte "01..S"

results_msg:
  .byte "Low-level controller port",10
  .byte "probing results",10
  .byte "Reset: return; Start: help",10
  .byte "",10
  .byte "PPU readback",10
  .byte "",10
  .byte "PPU latch",10
  .byte "APU open bus",10
  .byte "",10
  .byte "       4L 3L 4H 3H D76543210",10
  .byte "1P:",10
  .byte "2P:",0

page1_msg:
  .byte "To list controllers plugged",10
  .byte "into a Famicom or NES",10
  .byte "Control Deck, a program can",10
  .byte "use differences in open bus",10
  .byte "behavior to tell which input",10
  .byte "bits are always off, always",10
  .byte "on, serial, or not connected",10
  .byte "at all (NC).",10
  .byte "4L xor 4H are serial;",10
  .byte "4L xor 3L are NC.",10
  .byte "",10
  .byte "[1/4]",0

page2_msg:
  .byte "Open bus occurs when nothing",10
  .byte "puts a 0 or 1 on a bit of",10
  .byte "the data bus during a read.",10
  .byte "Capacitance holds the old",10
  .byte "voltage in place for the CPU",10
  .byte "to use as the bit's value.",10
  .byte "Reading nonexistent memory,",10
  .byte "for example, usually leaves",10
  .byte "the instruction's last byte",10
  .byte "on the bus, which on 6502 is",10
  .byte "the address's high byte.",10
  .byte "[2/4]",0

page3_msg:
  .byte "But the PowerPak has",10
  .byte "interfered with open bus",10
  .byte "before.  Pull-up resistors",10
  .byte "added to solve a DMA problem",10
  .byte "made NC bits look always on.",10
  .byte "Mindscape games failed to",10
  .byte "see presses because they",10
  .byte "expected open bus on unused",10
  .byte "controller port bits.",10
  .byte "Some NES-on-a-chip clones",10
  .byte "also get open bus wrong.",10
  .byte "[3/4]",0

page4_msg:
  .byte 34,"PPU readback",34," tests reading",10
  .byte "VRAM through $2007, $3F07,",10
  .byte "$3F17, and $3F07 carried",10
  .byte "across open bus to $4007.",10
  .byte "It should alternate 00 FF.",10
  .byte 34,"PPU latch",34," tests a separate",10
  .byte "data bus inside the PPU.",10
  .byte "All bytes should be 3F.",10
  .byte 34,"APU open bus",34," reads the",10
  .byte "write-only Pulse 2 period.",10
  .byte "These should be 40 40 3F.",10
  .byte "[4/4]Press Start for results",0

